--!parent
--!encoding=utf-8

--****************************************************************************
-- @file      LSW CORE LIB: $.importER
-- @author    Valentin Schmidt
-- @version   0.3
-- @desc
-- Allows to $.import assets at runtime. Supported file formats are:
-- bitmap:      bmp, jpg, png, tif, psd, gif
-- sound:       wav, mp3, ogg, aac (mp4-container only)
-- text:        txt, rtf, html
-- animation:   gif, swf
-- vectorshape: eps, ai -- MOVED TO $.eps!
--****************************************************************************

----------------------------------------
-- @constructor
-- @param {list} [tFormats]
----------------------------------------
on new (me, tFormats)
	$ = me.$
	
  if voidP(tFormats) then tFormats = []

  -- NOT RUNTIME-LOADABLE XTRAS

--  winXtras = []
--  macXtras = []
--
--  -- animation
--  if tFormats.getPos("animgif") then
--    winXtras.add("Animated GIF Asset.x32")
--    macXtras.add("Animated GIF Asset.xtra")
--  end if
--  
--  if tFormats.getPos("swf") then
--    winXtras.add("Flash Asset.x32")
--    macXtras.add("Flash Asset PPC.xtra")
--  end if
--  
--  if count(winXtras) OR count(macXtras) then
--    $.import("require").requireXtras(winXtras, macXtras)
--  end if

  -- RUNTIME-LOADABLE XTRAS

  --++++++++++++++++++++++++++++++++++++
  -- BITMAP
  --++++++++++++++++++++++++++++++++++++

  if tFormats.getPos("png") then
  	$.loadXtras(me.libdir, ["PNG Import Export.x32"], ["PNGImport.xtra"]) -- @todo: MAC ??
  end if

  if tFormats.getPos("tif") then
  	--$.loadXtras(me.libdir, ["TIFF Import Export.x32","Mix Services.x32"], [])
  	$.loadXtras(me.libdir, ["TIFF Import Export.x32"], ["TIFF Import Export.xtra"])
  end if

  if tFormats.getPos("psd") then
  	--$.loadXtras(me.libdir, ["Photoshop 3.0 $.import.x32","Mix Services.x32"], [])
  	$.loadXtras(me.libdir, ["Photoshop 3.0 $.import.x32"], ["Photoshop 3.0 Import.xtra"])
  end if

--  if tFormats.getPos("gif") then
--  	--$.loadXtras(me.libdir, ["Mix Services.x32"], [])
--  	$.loadXtras(me.libdir, [], [])
--  end if

  --++++++++++++++++++++++++++++++++++++
  -- SOUND
  --++++++++++++++++++++++++++++++++++++

  -- WAV

  -- AAC (MP4)
  if tFormats.getPos("aac") OR tFormats.getPos("mp4") then
  	$.loadXtras(me.libdir, ["MP4Asset.x32"], ["MP4Asset.xtra"])
  end if
  
  -- OGG
  if tFormats.getPos("ogg") then
  	$.loadXtras(me.libdir, ["Ogg Vorbis Import.x32"], [])
  end if
  
  -- MP3
  if tFormats.getPos("mp3") then
  	$.loadXtras(me.libdir, ["SWADCmpr.x32"], ["SWA Decompression PPC Xtra.xtra"])
  end if
  
  --++++++++++++++++++++++++++++++++++++
  -- TEXT
  --++++++++++++++++++++++++++++++++++++

  -- TEXT (TXT, RTF, HTM)
  if tFormats.getPos("txt") OR tFormats.getPos("rtf") OR tFormats.getPos("htm") then
    $.import("text")
  	$.import("file")
  end if

  --++++++++++++++++++++++++++++++++++++
  -- ANIMATION
  --++++++++++++++++++++++++++++++++++++

  -- ANIMGIF
  -- -> xtra not runtime-loadable, see above

  -- SWF
  -- -> xtra not runtime-loadable, see above

	-- must be loaded AFTER asset xtras!
	needsMixServives = ["gif","png","psd","tif","aac","mp4","ogg","mp3"]
	repeat with f in needsMixServives
		if tFormats.getPos(f) then
			$.loadXtras(me.libdir, ["Mix Services.x32"], ["Mix Services.xtra"])
			exit repeat
		end if
	end repeat
  return me
end

----------------------------------------
-- Loads asset file into cast
-- @param {string} tPath
-- @param {propList} [props]
-- @param {string} [forceType]
-- @return {member|false}
----------------------------------------
on load (me, tPath, props, forceType)
  if voidP(props) then props = [:]
  	
  if voidP(forceType) then
  	od = _player.itemdelimiter
  	_player.itemdelimiter = "."
  	forceType = the last item of tPath
  	_player.itemdelimiter = od
  end if
  
--  if forceType="animGif" then
--    m = new(#animGif)
--    m.linked = 1
--    m.filename = tPath
--    m.linked = 0
--    ok = (m.type=#animGif)

  if forceType="txt" then
    m = new(#text)
	  --m.bgColor=rgb(255,255,255)
		--m.color = rgb(0,0,0)
		--m.font = "Arial"
		--m.fontSize = 12
		--m.alignment = #left
    str = me.$.file.getString(tPath)
    ok = stringP(str)
    if ok then m.text = str

  else if forceType="rtf" then
    m = new(#text)
    str = me.$.file.getString(tPath)
    ok = stringP(str)
    if ok then m.rtf = str
   	
  else if forceType="htm" then
    m = new(#text)
    str = me.$.file.getString(tPath)
    ok = stringP(str)
    if ok then m.html = str
   	
  else
    m = new(#shape)
    ok = m.importFileInto(tPath, props)
  end if
  
  if ok then
    return m
  else
    m.erase()
    return false
  end if
end
